package com.steerinfo.dil.controller;

import com.steerinfo.dil.feign.TmsTruckFeign;
import com.steerinfo.dil.mapper.OmstruckOrderMapper;
import com.steerinfo.dil.mapper.OmstruckOrderSeparateMapper;
import com.steerinfo.dil.model.OmstruckOrder;
import com.steerinfo.dil.service.impl.OmstruckOrderSeparateServiceImpl;
import com.steerinfo.dil.util.BaseRESTfulController;
import com.steerinfo.dil.util.ColumnDataUtil;
import com.steerinfo.dil.util.DataChange;
import com.steerinfo.dil.util.PageListAdd;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageHelper;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * @ author    :TXF
 * @ time      :2021/12/3 15:07
 */

@RestController
@RequestMapping("/${api.version}/omstruckorderseparates")
public class OmstruckOrderSeparateController extends BaseRESTfulController {

    @Autowired
    OmstruckOrderSeparateMapper omstruckOrderSeparateMapper;

    @Autowired
    private OmstruckOrderMapper omstruckOrderMapper;

    @Autowired
    OmstruckOrderSeparateServiceImpl omstruckOrderSeparateService;

    @Autowired
    private TmsTruckFeign tmsTruckFeign;

    @Autowired
    ColumnDataUtil columnDataUtil;


    private final SimpleDateFormat sdfDateTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @PostMapping("/getOrderMes")
    public RESTfulResult getMes(@RequestBody(required = false) Map<String, Object> map){
        Integer integer = tmsTruckFeign.addTotalResult(map);
        return success(integer);
    }

    /**
     * 添加退货实绩关闭当前订单
     * @param map
     * @return
     */
    @PostMapping("/returnOrderCloseOrder")
    public RESTfulResult returnOrderCloseOrder(@RequestBody(required = false) Map<String, Object> map){
        Integer result = omstruckOrderSeparateService.returnOrderCloseOrder(map);
        return success(result);
    }

    /**
     * 根据运单id查询实绩位置
     * @param map
     * @return
     */
    @PostMapping("/getOrderResult")
    public List<Map<String,Object>> getOrderResult(@RequestBody Map<String,Object> map){
        BigDecimal orderId = DataChange.dataToBigDecimal(map.get("orderId"));
        String orderNumber = (String) map.get("orderNumber");
        List<Map<String, Object>> result = null;
        if (orderId != null && orderId.intValue() != 0) {
            result = omstruckOrderSeparateService.getOrderResult(orderId);
        }
        else if(orderNumber != null) {
            result = omstruckOrderSeparateService.getOrderNumberResult(orderNumber);
        }
        return result;
    }

    /**
     * 发送消息到 websocket 推送消息
     * @param map
     */
    @PostMapping("/pushMesToWebsocket")
    public RESTfulResult pushMesToWebsocket(@RequestBody Map<String,Object> map) {
        String s = omstruckOrderSeparateService.pushMesToWebsocket(map);
        return success(s);
    }

    @ApiOperation(value = "查看承运商所属的订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "mapValue", value = "表头和参数", required = false, dataType = "map"),
            @ApiImplicitParam(name = "apiId(86)", value = "动态表头", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageNum", value = "页码", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "页", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "status", value = "状态码", required = false, dataType = "Integer"),
    })
    @PostMapping("/getInputOrderList")
    public RESTfulResult getInputOrderList(@RequestBody(required = false) Map<String, Object> mapValue,
                                          Integer apiId,
                                          Integer pageNum,
                                          Integer pageSize,
                                          Integer carrierId,
                                          Integer orderType,
                                          Integer orderStatus,
                                          String  con) {
        if (orderStatus != null) {
            mapValue.put("orderStatus", orderStatus);
        }
        if (orderType != null) {
            mapValue.put("orderType", orderType);
        }
        if (carrierId != null) {
            mapValue.put("carrierId", carrierId);
        }
        PageHelper.startPage(pageNum, pageSize);
        //分页数据
        List<Map<String, Object>> truckOrder = omstruckOrderSeparateService.getInputOrderList(mapValue);
        PageListAdd pageList = columnDataUtil.tableColumnData(apiId, null, truckOrder);
        return success(pageList);
    }

    @ApiOperation(value = "查询所有的零星物资订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "mapValue", value = "表头和参数", required = false, dataType = "map"),
            @ApiImplicitParam(name = "apiId(86)", value = "动态表头", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageNum", value = "页码", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "页", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "orderStatus", value = "状态码", required = false, dataType = "Integer"),
    })
    @PostMapping("/getOthersOrderMesToSend")
    public RESTfulResult getOthersOrderMesToSend(@RequestBody(required = false) Map<String, Object> mapValue,
                                                 Integer apiId,
                                                 Integer pageNum,
                                                 Integer pageSize,
                                                 Integer type,
                                                 Integer orderStatus,
                                                 String carrierId,
                                                 String  con,
                                                 String userId,
                                                 String usersId,
                                                 Integer type1,
                                                 String startTime,
                                                 String endTime
                                                 ) {
        DataChange.queryDataByDateTime(startTime, endTime, mapValue,sdfDateTime);//根据时间段查询数据
        if (orderStatus != null) {
            mapValue.put("status", orderStatus);
        }
        if (type != null) {
            mapValue.put("type", type);
        }
        if (type1!=null){
            mapValue.put("type1",type1);
        }
        if (carrierId != null && !"undefined".equals(carrierId)) {
            mapValue.put("carrierId", carrierId);
        }
        if (userId!=null){
            mapValue.put("userId",userId);
        }
        if (usersId!=null){
            mapValue.put("usersId",usersId);
        }
        if(con != null && !"null".equals(con)){
            mapValue.put("con","%" + con + "%");
        }
        //框计算
        PageHelper.startPage(pageNum, pageSize);
        //分页数据
        List<Map<String, Object>> mes = omstruckOrderSeparateService.getOthersOrderMesToSend(mapValue);
        PageListAdd pageList = columnDataUtil.tableColumnData(apiId, null, mes);
        return success(pageList);
    }

    /**
     * 根据运输订单id修改运力id
     * @param omstruckOrder
     */
    @PostMapping("/updateOmstruckOrder")
    public RESTfulResult updateOmstruckOrder(@RequestBody(required = false) OmstruckOrder omstruckOrder){
        int i = omstruckOrderSeparateService.updateOmstruckOrder(omstruckOrder);
        return success(i);
    }

    @PostMapping("getUnloadOrderMaterial")
    public RESTfulResult getUnloadOrderMaterial(@RequestBody(required = false) Map<String,Object>map) {
        return success(omstruckOrderSeparateService.getUnloadOrderMaterial(map));
    }

    @PostMapping("getUnloadOrderWarehouseName")
    public RESTfulResult getUnloadOrderWarehouseName(@RequestBody(required = false) Map<String,Object> map){
        return success(omstruckOrderSeparateService.getUnloadOrderWarehouseName(map));
    }

    /**
     * 承运商销售分派运输订单给司机
     * @param mapValue
     * @return
     */
    @PostMapping("/addSaleTruckOrder")
    public RESTfulResult addSaleTruckOrder(@RequestBody Map<String, Object> mapValue){
        Integer result = omstruckOrderSeparateService.addSaleTruckOrder(mapValue);
        return success(result);
    }

    @ApiOperation(value = "该车牌号下的所有订单号 以及 订单ID")
    @PostMapping("/getCapacityAllOrder")
    public RESTfulResult getCapacityAllOrder(String capacityNumber,String date){
        Map<String, Object> map = new HashMap<>();
        map.put("capacityNumber",capacityNumber);
        try {
            if(date != null){
                date += "-01 00:00:00";
                DateTimeFormatter sdf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                LocalDateTime ldt = LocalDateTime.parse(date,sdf);
                int month = ldt.getMonth().getValue();
                int year = ldt.getYear();
                // 判断是否是一月，一月则减一年，月份回到12
                if (month == 1) {
                    map.put("startDate", (year - 1) + "-" + 12 + "-" + 26);
                }
                // 非一月只需减少一个月份
                if (month != 1){
                    if (month > 10) {
                        map.put("startDate", year + "-" + (month - 1) + "-" + 26);
                    }
                    else {
                        map.put("startDate", year + "-0" + (month - 1) + "-" + 26);
                    }
                }
                if (month >= 10) {
                    // 结束时间为当月25号
                    map.put("endDate", year + "-" + month + "-" + 25);
                }
                if (month < 10) {
                    // 结束时间为当月25号
                    map.put("endDate", year + "-0" + month + "-" + 25);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        List<Map<String, Object>> capacityAllOrderList = omstruckOrderSeparateService.getCapacityAllOrder(map);
        return success(capacityAllOrderList);
    }

    @ApiOperation(value = "该车牌号下的该月账期的物资数量")
    @PostMapping("/getMatrialWeight")
    public RESTfulResult getMatrialWeight(String capacityNumber,String date){
        Map<String, Object> map = new HashMap<>();
        date += "-01 00:00:00";
        map.put("capacityNumber",capacityNumber);
        DateTimeFormatter sdf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime ldt = LocalDateTime.parse(date,sdf);
        try {
            if(date != null){
                int month = ldt.getMonth().getValue();
                int year = ldt.getYear();
                // 判断是否是一月，一月则减一年，月份回到12
                if (month == 1) {
                    map.put("startDate", (year - 1) + "-" + 12 + "-" + 26);
                }
                // 非一月只需减少一个月份
                if (month != 1){
                    if (month > 10) {
                        map.put("startDate", year + "-" + (month - 1) + "-" + 26);
                    }
                    else {
                        map.put("startDate", year + "-0" + (month - 1) + "-" + 26);
                    }
                }
                if (month >= 10) {
                    // 结束时间为当月25号
                    map.put("endDate", year + "-" + month + "-" + 25);
                }
                if (month < 10) {
                    // 结束时间为当月25号
                    map.put("endDate", year + "-0" + month + "-" + 25);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        List<Map<String, Object>> capacityAllOrderList = omstruckOrderSeparateService.getCapacityAllOrderTwo(map);
        List<Map<String,Object>> matrialList=new LinkedList<>();
        //第二种解决思路
        for (Map<String, Object> stringObjectMap : capacityAllOrderList) {
            String materialName = (String) stringObjectMap.get("materialName");
            //因为netWeight有String类型的
            if (stringObjectMap.get("netWeight")==null){
                continue;
            }
            String netWeight1 = (String) stringObjectMap.get("netWeight");
            String[] split = netWeight1.split("、");
            if (split.length<=1){
                if (contains(matrialList,materialName)){
                    //把值给存入进去
                    addSomeThing(matrialList,materialName,DataChange.dataToBigDecimal(split[0]));
                }else {
                    if (split==null){
                        continue;
                    }
                    Map<String,Object> map1=new HashMap<>();
                    map1.put("materialName",materialName);
                    map1.put("netWeight",DataChange.dataToBigDecimal(split[0]));
                    matrialList.add(map1);
                }
            }else{
                for (String s : split) {
                    if (contains(matrialList,materialName)){
                        //把值给存入进去
                        addSomeThing(matrialList,materialName,DataChange.dataToBigDecimal(s));
                    }else {
                        if (s==null){
                            continue;
                        }
                        Map<String,Object> map1=new HashMap<>();
                        map1.put("materialName",materialName);
                        map1.put("netWeight",DataChange.dataToBigDecimal(s));
                        matrialList.add(map1);
                    }
                }
            }

        }
        //把物资的值都给求出来并放到数据中去
        //第一种想法
        /*for (Map<String, Object> stringObjectMap : capacityAllOrderList) {
            BigDecimal orderId = DataChange.dataToBigDecimal(stringObjectMap.get("orderId"));
            Integer orderType = omstruckOrderMapper.getOrderTypeByOrderNumber(orderId);
            String steelJudge = "steel";
                List<Map<String, Object>> materialMes = omstruckOrderMapper.getMaterialMesForGrossWeight(orderId,steelJudge);
                for (Map<String, Object> materialMe : materialMes) {
                    String materialName = (String) materialMe.get("materialName");
                    BigDecimal netWeight = DataChange.dataToBigDecimal(materialMe.get("netWeight"));
                    if (contains(matrialList,materialName)){
                        //把值给存入进去
                        addSomeThing(matrialList,materialName,netWeight);
                    }else {
                        if (netWeight.compareTo(BigDecimal.ZERO)==0){
                            continue;
                        }
                        Map<String,Object> map1=new HashMap<>();
                        map1.put("materialName",materialName);
                        map1.put("netWeight",netWeight);
                        matrialList.add(map1);
                    }
                    }*/
        return success(addChinese(matrialList));
    }

    //给每个值的后面加个吨字
    private List<Map<String,Object>> addChinese(List<Map<String, Object>> matrialList) {
        for (Map<String, Object> map : matrialList) {
            map.put("netWeight",map.get("netWeight")+"吨");
        }
        return matrialList;
    }

    private boolean contains(List<Map<String,Object>> matrialList,String materialName){
        if (matrialList.isEmpty()){
            return false;
        }
        for (Map<String, Object> map : matrialList) {
            if (map.get("materialName").equals(materialName)){
                return true;
            }
        }
        return false;
    }

    private void addSomeThing(List<Map<String,Object>> matrialList,String materialName,BigDecimal netWeight){
        if (netWeight.compareTo(BigDecimal.ZERO)==0){
            return;
        }
        for (Map<String, Object> map : matrialList) {
            if (map.get("materialName").equals(materialName)){
                BigDecimal weight= DataChange.dataToBigDecimal(map.get("netWeight"));
                map.put("materialName",materialName);
                map.put("netWeight",weight.add(netWeight));
                return;
            }
        }
        return;
    }

    @ApiOperation(value = "根据运单id关闭运输订单")
    @PostMapping("/closeOrder")
    public RESTfulResult closeOrder(@RequestParam Integer orderId){
        int result = omstruckOrderSeparateService.closeOrder(orderId);
        return success(result);
    }

    @ApiOperation(value = "根据运单id查询出所有销售数据")
    @PostMapping("/getSaleAllMessages")
    public Map<String,Object> getSaleAllMessages(@RequestParam Integer orderId){
        Map<String, Object> mapValue = omstruckOrderSeparateService.getSaleAllMessages(new BigDecimal(orderId));
        return mapValue;
    }

    @ApiOperation(value = "新增一个新的采购运输订单（一车多趟）")
    @PostMapping("/addNewTransportOrder")
    public RESTfulResult addNewTransportOrder(@RequestParam Integer orderId){
        int i = omstruckOrderSeparateService.addNewTransportOrder(new BigDecimal(orderId));
        return success(i);
    }

    @ApiOperation(value = "一键删除多个订单")
    @PostMapping("/deleteOrders")
    public RESTfulResult deleteOrders(@RequestBody (required = false) Map<String, Object> map){
        int i = omstruckOrderSeparateService.deleteOrders(map);
        return success(i);
    }

    @ApiOperation(value = "装卸工查询自己所有装了货的订单信息")
    @PostMapping("/getAllLoadUnloadResult")
    public RESTfulResult getAllLoadUnloadResult(@RequestBody (required = false) Map<String, Object> map){
        List<Map<String, Object>> list = omstruckOrderSeparateService.getAllLoadUnloadResult(map);
        return success(list);
    }



    @ApiOperation(value = "装机查询所以已装车的订单信息")
    @PostMapping("/getLoadForLoadCarResult")
    public RESTfulResult getLoadForLoadCarResult(@RequestBody (required = false) Map<String, Object> map){
        List<Map<String, Object>> list = omstruckOrderSeparateService.getLoadForLoadCarResult(map);
        return success(list);
    }


    //根据userId查找改运力是否为装机

    @ApiOperation(value = "装机查询所以已装车的订单信息")
    @PostMapping("/getCapacityTypeByUserId")
    public RESTfulResult getCapacityTypeByUserId(@RequestBody (required = false) Map<String, Object> map){
        Integer capacityTypeByUserId = omstruckOrderSeparateService.getCapacityTypeByUserId(map);
        return success(capacityTypeByUserId);
    }


    @ApiOperation(value = "查看计时完成详情")
    @PostMapping("/getTimeTaskEnd")
    public RESTfulResult getTimeTaskEnd(@RequestBody (required = false) Map<String, Object> map){
        return success(omstruckOrderSeparateService.getTimeTaskEnd(map));
    }
    @ApiOperation(value = "查看计时完成详情")
    @PostMapping("/getTimeTaskEndDetail")
    public RESTfulResult getTimeTaskEndDetail(@RequestBody (required = false) Map<String, Object> map){
        return success(omstruckOrderSeparateService.getTimeTaskEndDetail(map));
    }

    @ApiOperation(value = "查看暂停时长")
    @PostMapping("/getPauseTimeTaskDetail")
    public RESTfulResult getPauseTimeTaskDetail(@RequestParam  Integer orderId){
        return success(omstruckOrderSeparateService.getPauseTimeTaskDetail(orderId));
    }


    @ApiOperation(value = "批量关闭运输订单")
    @PostMapping("/batchCloseOrder")
    public RESTfulResult batchCloseOrder(@RequestBody List<Map<String,Object>> mapList){
        return success(omstruckOrderSeparateService.batchCloseOrder(mapList));
    }

    @ApiOperation(value = "根据订单ID查询订单子表物资信息")
    @PostMapping("/getOrderMaterial/{orderId}")
    public RESTfulResult getOrderMaterial(@PathVariable("orderId")Integer orderId) {
        return success(omstruckOrderSeparateService.getOrderMaterial(orderId));
    }


    @ApiOperation(value = "正常关闭运输订单")
    @PostMapping("/closeInwardOrder")
    public RESTfulResult closeInwardOrder(Integer orderId, Integer sqe, String resultNetWeight){
        return success(omstruckOrderSeparateService.closeInwardOrder(orderId, sqe, resultNetWeight));
    }

    @PostMapping("testJarSuccess")
    public RESTfulResult testJarSuccess() {
        return success("jar包发布成功");
    }

}
